#include "PLDSetupAssistant.h"

#include <QLabel>
#include <QTabWidget>
#include <QVBoxLayout>

#include <KActionMenu>
#include <KCMultiDialog>
#include <KDebug>
#include <KLocale>
#include <KPushButton>
#include <KServiceTypeTrader>
#include <KStandardDirs>
#include <KTitleWidget>
#include <KVBox>

PLDSetupAssistant::PLDSetupAssistant( QWidget *parent )
  : KDialog( parent )
{
  
    setPlainCaption(i18n("PLD Setup Assistant"));
    setButtons(User1 | Close);
    setButtonGuiItem(User1 , KGuiItem( i18n( "Start Assistant" ), "tools-wizard", i18n("Starts the Assistant")));
    connect(this, SIGNAL(user1Clicked()), this, SLOT(startAssistant()));
    showButtonSeparator(true);
    //setButtons(KDialog::Close);

    KTitleWidget *titleWidget = new KTitleWidget(this);
    titleWidget->setText(i18n("<html><font size=\"5\">Welcome to PLD Linux Setup Assistant</font><br /><b>Version 0.1</b></html>"));
    titleWidget->setPixmap(KIcon("pld_czarny").pixmap(48), KTitleWidget::ImageLeft);
    
    // this PNG should be installed into /usr/share/apps/pldsetupassistant/
    QLabel *logo = new QLabel;
    logo->setPixmap(KStandardDirs::locate("data", "pldsetupassistant/pld_conf.png"));

    QVBoxLayout *logoLayout = new QVBoxLayout;
    logoLayout->addWidget(logo);
    
    //KHBox *buttonBox = new KHBox;
    /*
    nextButton = new KPushButton(KIcon("tools-wizard"), i18n("Start Assistant"), buttonBox);
    nextButton->setSizePolicy(QSizePolicy::Fixed,QSizePolicy::Fixed);
    nextButton->setFlat(true);
    nextButton->setToolTip("Starts the Assistant");
    connect(nextButton, SIGNAL(clicked()), this, SLOT(startAssistant()));
    closeButton = new KPushButton(KStandardGuiItem::close(), buttonBox);
    closeButton->setSizePolicy(QSizePolicy::Fixed,QSizePolicy::Fixed);
    closeButton->setFlat(true);
    connect( closeButton, SIGNAL(clicked()), this, SLOT( close() ));
    */
    
    QVBoxLayout *mainLayout = new QVBoxLayout;
    mainLayout->addWidget(titleWidget);
    mainLayout->addLayout(logoLayout);
    //mainLayout->addWidget(buttonBox);
    mainLayout->setMargin(0);

    QWidget *mainWidget = new QWidget;
    mainWidget->setLayout(mainLayout);
    mainWidget->setSizePolicy(QSizePolicy::Fixed,QSizePolicy::Fixed);

    setMainWidget(mainWidget);
    
}

PLDSetupAssistant::~PLDSetupAssistant()
{
}

void PLDSetupAssistant::startAssistant()
{
    delayedDestruct();
    StartAssistant assistant;
    assistant.exec();
}

StartAssistant::StartAssistant( QWidget *parent )
  : KAssistantDialog( parent )
{
    setPlainCaption(i18n("PLD Setup Assistant"));
    showButton(KDialog::Help, false);
    
    // Keyboard
    keyboardDialog = kcmDialog("kcm_keyboard_layout", "keyboard_layout");
    keyboardPage = addPage(kcmLayout(keyboardDialog), i18n("Step One: Choose your keyboard layout"));
    if (!keyboardDialog) setAppropriate(keyboardPage, false);

    // Date and Time
    clockDialog = kcmDialog("kcm_clock", "clock");
    clockPage = addPage(kcmLayout(clockDialog), i18n("Step Two: Choose your timezone and setup the clock"));
    if (!clockDialog) setAppropriate(clockPage, false);
    
    // Network
    netDialog = kcmDialog("kcm_networkmanagement", "kcm_networkmanagement");
    netPage = addPage(kcmLayout(netDialog), i18n("Step Three: Setup your network"));
    if (!netDialog) setAppropriate(netPage, false);
    
    // Finish
    finishDialog = finish();
    finishPage = addPage(finishDialog, i18n(""));

}

StartAssistant::~StartAssistant()
{
}

QWidget * StartAssistant::finish()
{
    KTitleWidget *titleWidget = new KTitleWidget(this);
    titleWidget->setText(i18n("<html><font size=\"5\">Hurray, the setup is now complete!</font>"
	  "</html>"));
    titleWidget->setPixmap(KIcon("pld_czarny").pixmap(48), KTitleWidget::ImageLeft);
    
    QLabel *authors = new QLabel;
    authors->setMargin(10);
    authors->setAlignment(Qt::AlignTop);
    authors->setWordWrap(true);
    authors->setOpenExternalLinks(true);
    authors->setTextInteractionFlags(Qt::TextBrowserInteraction);
    authors->setText(i18n("<html>"
	  "Please use <a href=\"https://bugs.launchpad.net/pld-linux\">https://bugs.launchpad.net/pld-linux</a>"
	  " to report bugs.<br /><br />"
	  "Bartosz Świątek<br />"
	  "   <a href=\"mailto:shadzik@pld-linux.org\">shadzik@pld-linux.org</a><br />"
	  "   Maintainer, Qt4 programming<br /><br />"
	  "Bartłomiej Zimoń<br />"
	  "   <a href=\"mailto:cactus@pld-linux.org\">cactus@pld-linux.org</a><br />"
	  "   Qt4 programming"
	  "</html>"));
	  
    QLabel *thanks = new QLabel;
    thanks->setMargin(10);
    thanks->setAlignment(Qt::AlignTop);
    thanks->setWordWrap(true);
    thanks->setOpenExternalLinks(true);
    thanks->setTextInteractionFlags(Qt::TextBrowserInteraction);
    thanks->setText(i18n("<html>"
	  "Karol Kreński<br />"
	  "   <a href=\"mailto:mimooh@pld-linux.org\">mimooh@pld-linux.org</a><br />"
	  "   Graphics"
	  "</html>"));
	  
    QTabWidget *tabWidget = new QTabWidget;
    tabWidget->setUsesScrollButtons(false);
    tabWidget->addTab(authors, i18nc("Authors","&Authors"));
    tabWidget->addTab(thanks, i18nc("Thanks To","&Thanks"));
    
    QLabel *image = new QLabel;
    image->setPixmap(KStandardDirs::locate("data", "pldsetupassistant/pld_wizard.png"));

    QHBoxLayout *midLayout = new QHBoxLayout;
    midLayout->addWidget(image);
    midLayout->addWidget(tabWidget);

    QVBoxLayout *layout = new QVBoxLayout(this);
    layout->addWidget(titleWidget);
    layout->addLayout(midLayout);
    layout->setMargin(0);
    
    QWidget *widget = new QWidget(this);
    widget->setLayout(layout);
    widget->setSizePolicy(QSizePolicy::Fixed,QSizePolicy::Fixed);
    return widget;
}

QWidget * StartAssistant::kcmLayout(KCMultiDialog* dialog)
{
    QVBoxLayout *layout = new QVBoxLayout(this);
    layout->addWidget(dialog);
    layout->setMargin(0);

    QWidget *widget = new QWidget(this);
    widget->setLayout(layout);
    widget->setSizePolicy(QSizePolicy::Fixed,QSizePolicy::Fixed);
    return widget;
}

KCMultiDialog* StartAssistant::kcmDialog(const QString lib, const QString name, bool debug)
{
  KService::List serviceList = KServiceTypeTrader::self()->query("KCModule");
  serviceList.append(KServiceTypeTrader::self()->query("SystemSettingsCategory"));

  foreach (const KService::Ptr service, serviceList) {
    if (service->library() == lib && ( name.isEmpty() || service->desktopEntryName() == name))
    {
      if (debug) kDebug() <<service->library() << service->desktopEntryName() << service->name();
      KCMultiDialog *dlg = new KCMultiDialog(this);
      dlg->addModule(KCModuleInfo(service));
      dlg->showButton(KDialog::Apply, false);
      dlg->showButton(KDialog::Ok, false);
      dlg->showButton(KDialog::Cancel, false);
      dlg->showButton(KDialog::Help, false);
      dlg->showButton(KDialog::Default, false);
      dlg->showButtonSeparator(false);
      return dlg;
    } else if (debug) kDebug() <<service->library() << service->desktopEntryName() << service->name();
  }

  return 0;
}

void StartAssistant::back()
{
      KAssistantDialog::back();
}

void StartAssistant::next()
{
      if (currentPage() == keyboardPage)
	keyboardDialog->button(KDialog::Apply)->click();

      if (currentPage() == clockPage)
	clockDialog->button(KDialog::Apply)->click();
      
      if (currentPage() == netPage)
	netDialog->button(KDialog::Apply)->click();

      KAssistantDialog::next();
}

#include "PLDSetupAssistant.moc"
